function [ CMFs_IndivXYZ, CMFs_IndivRGB, list_Age, t_raw ] = getObs_SBIndivCMFs( fieldSize )

%{
xls files from http://cvrl.ioo.ucl.ac.uk/
In SB10_individual_CMFs.xls, Obs37 & Obs43 have NaN values
So they are interpolated manually by Yuta
Obs37: Nearest-neighbor extrapolation
Obs43: Spline interpolation

Obs 1, 2, 4, 24 repeated experiment twice, they appear as obs 32, 52, 45, 44
These four observers data are averaged

%}


if nargin == 0
    fieldSize = 10; % default:10-deg
end

cie = getCieStruct(380:1:780);


if fieldSize == 2
    
    % Load File
    n_obs = 10; 
    t_raw = xlsread(['SB' num2str(fieldSize) '_individual_CMFs.xls']); % 2-deg: 392.16nm - 727.27nm
    t_CMFsIndivRGB = interp1(t_raw(:,1),t_raw(:,3:32),380:1:780,'spline');

    % Get Age (Modify if necessary)
    list_Age = [53, 28, 29, 20, 30, 27, 25, 47, 20, 25]; % directly from excel sheet
    
    % Modify Extrapolation
    wv_target = 392;
    idx_wv = wv_target-380+1;
    t_CMFsIndivRGB( 1:idx_wv , : ) = repmat( t_CMFsIndivRGB(idx_wv,:), length(380:wv_target), 1);
    
    wv_target = 727;
    idx_wv = wv_target-380+1;
    t_CMFsIndivRGB( idx_wv:end , : ) = repmat( t_CMFsIndivRGB(idx_wv,:), length(wv_target:780), 1);
    
    % Sort
    for k_obs = 1:n_obs
        CMFs_IndivRGB(:,1,k_obs) = t_CMFsIndivRGB(:,3*(k_obs-1)+1);
        CMFs_IndivRGB(:,2,k_obs) = t_CMFsIndivRGB(:,3*(k_obs-1)+2);
        CMFs_IndivRGB(:,3,k_obs) = t_CMFsIndivRGB(:,3*(k_obs-1)+3);
    end
    
    % Linear Transformation
    cmf_ref = cie.cmf2deg;
    for k_obs = 1:n_obs
        % [ cmf_out , M ] = cmfA2cmfB( cmf_in , cmf_ref )
        [ CMFs_IndivXYZ(:,:,k_obs) , ~ ] = cmfA2cmfB( CMFs_IndivRGB(:,:,k_obs) , cmf_ref );
    end
    
    
    
    
elseif fieldSize == 10
    
    n_obs = 49; % either 49 or 53
        
    % Load File
    t_raw_53Obs = xlsread(['SB' num2str(fieldSize) '_individual_CMFs.xls']); % 10-deg: 392.16nm - 714.29nm
    
    % index for repeated obs
    idx_repObs1 = [1 2 4 24]; 
    idx_repObs2 = [32 52 45 44]; 
    
    % Data for repeated obs
    for k = 1:4
        t_idx1 = 3*( idx_repObs1(k) -1) + 1 + 2; 
        t_raw_repObs1(:,:,k) = t_raw_53Obs(:,t_idx1:t_idx1+2); 
        
        t_idx2 = 3*( idx_repObs2(k) -1) + 1 + 2; 
        t_raw_repObs2(:,:,k) = t_raw_53Obs(:,t_idx2:t_idx2+2); 
    end
    t_raw_repObs = (t_raw_repObs1+t_raw_repObs2)/2; 
    
    % Overwrite repeated obs data & Remove repeated obs
    t_raw_49Obs = t_raw_53Obs; 
    for k = 1:4
        t_idx1 = 3*( idx_repObs1(k) -1) + 1 + 2; 
        t_raw_49Obs(:,t_idx1:t_idx1+2) = t_raw_repObs(:,:,k); 
        
        t_idx2 = 3*( idx_repObs2(k) -1) + 1 + 2; 
        t_raw_49Obs(:,t_idx2:t_idx2+2) = nan; 
    end
    idx_Nan = isnan(t_raw_49Obs(8,:)); 
    t_raw_49Obs = t_raw_49Obs(:,~idx_Nan); 
    
    
    % Determine which data (49Obs/53Obs) is used 
    if n_obs == 49
        t_raw = t_raw_49Obs; 
    elseif n_obs == 53
        t_raw = t_raw_53Obs; 
    end
    
    
    % Interpolation
    t_CMFsIndivRGB = interp1(t_raw(8:end,2),t_raw(8:end,3:end),380:1:780,'spline');
    
    
    % Get Age
%     t_list_Age = t_raw(4,:); 
%     t_list_Age = t_list_Age(~isnan(t_list_Age));     
%     t_list_Age_repObs = ( t_list_Age(idx_repObs1) + t_list_Age(idx_repObs2) ) / 2; % list_age for repeated obs
%     t_list_Age(idx_repObs2) = nan; % remove repeated obs
%     t_list_Age(idx_repObs1) = t_list_Age_repObs; % use average age for repeated obs
%     t_list_Age = t_list_Age(~isnan(t_list_Age)); 
    
    % Get Age
    list_Age = t_raw(4,:); 
    list_Age = list_Age(~isnan(list_Age)); 


    % Modify Extrapolation
    wv_target = 392; % short-wv region (left-end)
    idx_wv = wv_target-380+1;
    t_CMFsIndivRGB( 1:idx_wv , : ) = repmat( t_CMFsIndivRGB(idx_wv,:), length(380:wv_target), 1);
    
    wv_target = 714; % long-wv region (right-end)
    idx_wv = wv_target-380+1;
    t_CMFsIndivRGB( idx_wv:end , : ) = repmat( t_CMFsIndivRGB(idx_wv,:), length(wv_target:780), 1);
    
    % Sort
    for k_obs = 1:n_obs
        CMFs_IndivRGB(:,1,k_obs) = t_CMFsIndivRGB(:,3*(k_obs-1)+1);
        CMFs_IndivRGB(:,2,k_obs) = t_CMFsIndivRGB(:,3*(k_obs-1)+2);
        CMFs_IndivRGB(:,3,k_obs) = t_CMFsIndivRGB(:,3*(k_obs-1)+3);
    end
    
%     % Remove Repeated Observers
%     CMFs_IndivRGB = CMFs_IndivRGB53Obs; 
%     CMFs_IndivRGB_repObs = ( CMFs_IndivRGB53Obs(:,:,idx_repObs1) + CMFs_IndivRGB53Obs(:,:,idx_repObs2) ) / 2; % CMFs_RGB for repeated obs
%     CMFs_IndivRGB(:,:,idx_repObs2) = nan; % Remove repeated obs
%     CMFs_IndivRGB(:,:,idx_repObs1) = CMFs_IndivRGB_repObs; % Use average CMFs for repeated obs
%     CMFs_IndivRGB = CMFs_IndivRGB(~isnan(CMFs_IndivRGB)); 
    
    % Linear Transformation
    cmf_ref = cie.cmf10deg;
    for k_obs = 1:n_obs
        % [ cmf_out , M ] = cmfA2cmfB( cmf_in , cmf_ref )
        [ CMFs_IndivXYZ(:,:,k_obs) , ~ ] = cmfA2cmfB( CMFs_IndivRGB(:,:,k_obs) , cmf_ref );
    end
end

% figure; hold on; grid on; 
% for k = 1:49
%     plot(380:1:780, CMFs_IndivXYZ(:,:,k)); 
% end

end






