function [ x_stat ] = stat(x)
% x: m(observations whose mean you want to know) x n(sets)
%	output_row: max,90th percentile,mean,median,min,stdev
%	output_col: n(sets)



% If x is 3D, x is transformed to 2D
%   output_row: max,90th percentile,mean,median,min,stdev   for  dim1&dim2 of x
%   output_col: dim3 of x
if size(x,3) ~= 1
    x_3D = x; 
    x_2D = reshape( x_3D , size(x_3D,1)*size(x_3D,2) , size(x_3D,3) ); 
    
    % replace x with 2D data
    clear x; x = x_2D; 
end



if sum(sum(isnan(x))) == 0 % there is no NaN
    
    num_row = size(x,1);
    mean_x = sum(x)./num_row;
    max_x = max(x);
    min_x = min(x);
    stdev_x = std(x);
    pct90_x = prctile(x,90);
    median_x = median(x);
    
    x_stat = [max_x;pct90_x;mean_x;median_x;min_x;stdev_x]; 

    
else  % there is NaN
%     warning('input contains NaN, they are ignored.');
    x_stat = nan(6,size(x,2));
    for k = 1:size(x,2)
        x_indiv = x(:,k);
        
        idx_nan = isnan(x_indiv);
        idx_isnan = (idx_nan==0);
        x_newIndiv = x_indiv(idx_isnan);
        
        num_row = size(x_newIndiv,1); 
        
        if num_row == 0 % when all the observations are NaN
            x_stat(:,k) = nan(6,1); 
        else
            mean_x = sum(x_newIndiv)./num_row;
            max_x = max(x_newIndiv);
            min_x = min(x_newIndiv);
            stdev_x = std(x_newIndiv);
            pct90_x = prctile(x_newIndiv,90);
            median_x = median(x_newIndiv);
            
            x_stat(:,k) = [max_x;pct90_x;mean_x;median_x;min_x;stdev_x];
        end
    end
    
    
end

end