clear all; 
addpath(genpath('ColorToolbox')); 
addpath('Toolbox_IndivObs'); 
addpath('ObsCalib_Res'); 
addpath('Toolbox_ObsCalib'); 
cie = getCieStruct(380:1:780); 

%%%%%%%%%%%%%%%%%%%%%%%%
%%% Define Variables %%%
%%%%%%%%%%%%%%%%%%%%%%%%

% use n_population = 76 and n_rep = 100 for data validation, 
% use n_population = 1000 and n_rep = 1 for F-test
n_population = 76; 
n_rep = 1; 

fs = 8.5; 
cmf_target = cie.cmf10deg; 

%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%


%% Generate CMFs by Monte-Carlo Simulation

% Create Observer ID list (exclude unusual observers, color deficient observers)
obsList = [1:161]; 
[ list_obsID_ColDef ] = fnc_getSpecificObsID( 'Color Deficiency [Y/N]', 'Y' ); 
obsRemove = [119 120 125 140 list_obsID_ColDef]; 
obsList(obsRemove) = []; 
obsList(obsList>80) = []; % remove observers from Germany
n_obs = length(obsList); 

[ list_Age ] = fnc_readObsData( 'Age' ); 
list_Age = list_Age(obsList); 


for k_res = 1:n_rep

% CMFs
[LMS_All, var_age, vAll ] = fnc_genMonteCarloObs( n_population, list_Age, fs); 
for k = 1:n_population
    t_LMS = interp1(390:5:780, LMS_All(:,:,k), cie.lambda, 'spline');
    t_LMS(t_LMS<0) = 0;
    [ t_cmf , t ] = cmfA2cmfB( t_LMS , cmf_target );
    CMFs_All(:,:,k) = t_cmf;
end


%% Prepare SPD_match, SPD_ref

for k_obs = 1:n_obs
    obsID = obsList(k_obs);
    
    % Load Results
    matName = sprintf('ColMatchExpRes_Obs%03d.mat',obsID); load(matName);
    
    % Get LAB_res (3 trials average), expDate
    LAB_res(:,:,k_obs) = squeeze(nanmean(LAB_adj3Rep5Exp,2));
    
    expDataAllObs{k_obs} = expDate; 
    
    SPD_match5ExpAllObs(:,:,:,k_obs) = SPD_match5Exp; 
    SPD_ref5ExpAllObs(:,:,k_obs) = SPD_ref5Exp;
    maxmin_ab_5ExpAllObs(:,:,:,k_obs) = maxmin_ab_5Exp; 
end
clear SPD_match5Exp SPD_ref5Exp maxmin_ab_5Exp

% Get Experiment Setting
k_obs = 10; % use data for this obs
SPD_match5Exp = SPD_match5ExpAllObs(:,:,:,k_obs); 
SPD_ref5Exp = SPD_ref5ExpAllObs(:,:,k_obs); 
maxmin_ab_5Exp = maxmin_ab_5ExpAllObs(:,:,:,k_obs); 

%% Color Matching Simulation

def_cmf = cie.cmf10deg; 

LAB_prd = nan(3, 5, n_population); 
for k_exp = 1:5
    t_spd_match = SPD_match5Exp(:,:,k_exp);
    t_spd_ref = SPD_ref5Exp(:,k_exp);
    
    % Get ill
    if k_exp==1 || k_exp==2 || k_exp==5 % use refSPD as refW
        [ t_ill ] = calcIll( t_spd_ref, t_spd_ref, def_cmf, 50 );
    elseif k_exp==3 || k_exp==4 % use illE for colored refSPD
        [ t_ill ] = calcIll( t_spd_ref, cie.illE, def_cmf, 50 );
    end
    t_XYZn = 683*def_cmf'*t_ill;
    
    
    for k_cmf = 1:size(CMFs_All,3)
        t_cmf = CMFs_All(:,:,k_cmf);
        
        
        % Perform Color Matching for each observer
        t_scr = pinv( 683*t_cmf'*t_spd_match ) * 683*t_cmf'*t_spd_ref;
        t_scr(t_scr<0) = 0; t_scr(t_scr>1) = 1;
        
        % Calc CIELAB for def_cmf
        t_XYZ_prd = 683*def_cmf'*t_spd_match*t_scr;
        t_LAB_prd = XYZ2Lab(t_XYZ_prd,t_XYZn);
        
        
        % Limit a*b* values if needed
        t_max_ab = maxmin_ab_5Exp(:,1,k_exp); 
        t_min_ab = maxmin_ab_5Exp(:,2,k_exp); 
        for k_ab = 1:2
            if t_LAB_prd(k_ab+1) < t_min_ab(k_ab)
                t_LAB_prd(k_ab+1) = t_min_ab(k_ab); 
            elseif t_LAB_prd(k_ab+1) > t_max_ab(k_ab)
                t_LAB_prd(k_ab+1) = t_max_ab(k_ab); 
            end
        end
        
        LAB_prd(:,k_exp,k_cmf) = t_LAB_prd;
        
    end
end



%% Stats

stat_5Exp_a_res = stat(squeeze(LAB_res(2,:,:))'); 
stat_5Exp_b_res = stat(squeeze(LAB_res(3,:,:))'); 

stat_5Exp_a_prd = stat(squeeze(LAB_prd(2,:,:))'); 
stat_5Exp_b_prd = stat(squeeze(LAB_prd(3,:,:))'); 


stdev_5Exp_res = [stat_5Exp_a_res(6,:) ; stat_5Exp_b_res(6,:)]; 
stdev_5Exp_prd = [stat_5Exp_a_prd(6,:) ; stat_5Exp_b_prd(6,:)]; 

res_SD_prd(k_res) = mean(mean(stdev_5Exp_prd)); 



end

fprintf('The predicted standard deviation is %.2f\n', mean(res_SD_prd)); 


%% Stat Test

% Two-sample F-test to test if the variances of two samples are equal
% test performed for each of 10 dimensions (2*5)

n_dim = 2*5; 
t_F1 = reshape(LAB_res(2:3,:), n_dim, size(LAB_res,3)); 
t_F2 = reshape(LAB_prd(2:3,:), n_dim, size(LAB_prd,3)); 
Ftest_alpha = 0.05; 
% [Ftest_h, Ftest_p, Ftest_ci, Ftest_stat] = vartest2(t_F1', t_F2', 'Alpha',0.05, 'Tail','both');
[Ftest_h, Ftest_p, Ftest_ci, Ftest_stat] = vartest2(t_F1', t_F2', Ftest_alpha, 'both');

fprintf('Variances of two samples are different for %d cases!\n', sum(Ftest_h)); 
fprintf('Variances of two samples are equal for %d cases!\n', n_dim-sum(Ftest_h)); 









