clear all; 
addpath(genpath('ColorToolbox')); 
addpath('Toolbox_IndivObs'); 
cie = getCieStruct(380:1:780); 

%%%%%%%%%%%%%%%%%%%%%%%%
%%% Define Variables %%%
%%%%%%%%%%%%%%%%%%%%%%%%

def_RGYprm = [700 550 589];  
def_RGYwdthInNM = [1 1 1].*10^(-3); % for monochromatic light, use very small number (but not zero) 
def_lumY = 18.4187; 
outType = 'dE'; % 'MCDM' or 'dE'
dEtype = 'dE00'; 
def_cmf = cie.cmf2deg; 


n_population = 113; 
n_rep = 1; 
fs = 2; 

% Rufer_etal_2012
list_Age = [ones(1,15)*14.9 ones(1,38)*29.2 ones(1,33)*50.9 ones(1,27)*65.4]; % taken from Table 1
val_RG_target = [43.5, 2.7]; % mean and StdDev

%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%


% Get SPD_R, G, Y (normalize such that lum = def_lumY for all primaries)
SPD_RG = zeros(401,2); 
k_RG = 1; SPD_RG( def_RGYprm(k_RG)-380+1 ,k_RG) = 1; 
k_RG = 2; SPD_RG( def_RGYprm(k_RG)-380+1 ,k_RG) = 1; 

% [ SPD_gauss ] = fnc_getSPDgauss( peak, width, wv )
k_RGY = 1; 
SPD_RG(:,k_RGY) = getSPDgauss( def_RGYprm(k_RGY), def_RGYwdthInNM(k_RGY), cie.lambda ); 
SPD_RG(:,k_RGY) = def_lumY * SPD_RG(:,k_RGY) ./ ( 683*def_cmf(:,2)'*SPD_RG(:,k_RGY) ); % Set lum = def_lumY

k_RGY = 2; 
SPD_RG(:,k_RGY) = getSPDgauss( def_RGYprm(k_RGY), def_RGYwdthInNM(k_RGY), cie.lambda ); 
SPD_RG(:,k_RGY) = def_lumY * SPD_RG(:,k_RGY) ./ ( 683*def_cmf(:,2)'*SPD_RG(:,k_RGY) ); % Set lum = def_lumY

k_RGY = 3; 
SPD_Y = getSPDgauss( def_RGYprm(k_RGY), def_RGYwdthInNM(k_RGY), cie.lambda ); 
SPD_Y = def_lumY * SPD_Y ./ ( 683*def_cmf(:,2)'*SPD_Y ); 

% Get SPD of reference white (set lum = 100)
[ ill ] = calcIll( SPD_Y, ones(401,1), def_cmf, 50 ); 


for k_res = 1:n_rep

% CMFs
[LMS_All, var_age, vAll ] = fnc_genMonteCarloObs( n_population, list_Age, fs); 
for k = 1:n_population
    t_LMS = interp1(390:5:780, LMS_All(:,:,k), cie.lambda, 'spline');
    t_LMS(t_LMS<0) = 0;
    [ t_cmf , t ] = cmfA2cmfB( t_LMS , def_cmf );
    CMFs_All(:,:,k) = t_cmf;
end


%% Optimize coefficients of (1)yellow intensity & (2)ratio of R/G (amount of R, 0-1)

x0 = [1 0.5]'; %initial coefficients
A=[];
b=[];
Aeq=[]; beq=[]; 
lb=[0 0]'; 
ub=[10 1]'; 
nonlcon=[]; 
options = optimset('Display','none','MaxFunEvals',100*length(x0),'TolFun',1e-12,'Algorithm','active-set');
% default MaxFunEvals: 100*numberOfVariables
% default TolFun: 1e-6

coef_opt = nan(2,size(CMFs_All,3)); 
fval = nan(1,size(CMFs_All,3));
for k_cmf = 1:size(CMFs_All,3)
    [coef_opt(:,k_cmf),fval(k_cmf)] = fmincon(@obj_RayleighMatch,x0,A,b,Aeq,beq,lb,ub,nonlcon,options,...
        SPD_RG, SPD_Y, ill, CMFs_All(:,:,k_cmf), cie);
end

%% 

val_RG = coef_opt(2,:)*73; 
val_RG = val_RG - mean(val_RG) + val_RG_target(1); % adjust mean to fit the target dataset

res_SD_prd(k_res) = std(val_RG'); 
fprintf('k_res = %d is done!\n', k_res); 
end

fprintf('The predicted standard deviation is %.2f\n', mean(res_SD_prd)); 

