
%{

CMFs_RGB_SB are normalized by 
444.4nm (idx:7)
526.3nm (idx:19)
645.2nm (idx:32)

%}


clear all; 
addpath(genpath('ColorToolbox')); 
addpath('Toolbox_IndivObs'); 
cie = getCieStruct(380:1:780); 

%%%%%%%%%%%%%%%%%%%%%%%%
%%% Define Variables %%%
%%%%%%%%%%%%%%%%%%%%%%%%

% use n_population = 49 and n_rep = 100 for data validation, 
% use n_population = 1000 and n_rep = 1 for F-test
n_population = 49; 
n_rep = 1; 

fs = 10; 

%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%


%%

[ CMFs_IndivXYZ, ~, list_Age_SB, t_raw ] = getObs_SBIndivCMFs( 10 ); 
n_obs = length(list_Age_SB); 
CMFs_RGB_SBraw = t_raw(8:end,3:end); 
wv_SB = t_raw(8:end,2); 


for k_obs = 1:n_obs
    CMFs_RGB_SB(:,1,k_obs) = CMFs_RGB_SBraw(:,3*(k_obs-1)+1);
    CMFs_RGB_SB(:,2,k_obs) = CMFs_RGB_SBraw(:,3*(k_obs-1)+2);
    CMFs_RGB_SB(:,3,k_obs) = CMFs_RGB_SBraw(:,3*(k_obs-1)+3);
end
cmf_target = mean(CMFs_RGB_SB,3); 

res_SD_prd = []; 
for k_res = 1:n_rep

% CMFs
[LMS_All, var_age, vAll ] = fnc_genMonteCarloObs( n_population, list_Age_SB, fs); 
for k = 1:n_population
    t_LMS = interp1(390:5:780, LMS_All(:,:,k), wv_SB, 'spline');
    t_LMS(t_LMS<0) = 0;
    [ t_cmf , t ] = cmfA2cmfB( t_LMS , cmf_target );
    CMFs_All(:,:,k) = t_cmf;
end


for k = 1:n_population
    t_cmf = CMFs_All(:,:,k); 
    M_LMS2RGB = eye(3)*pinv([t_cmf(32,:)' t_cmf(19,:)' t_cmf(7,:)']); 
    CMFs_RGB_VisM(:,:,k) = (M_LMS2RGB*t_cmf')';
end


%% StdDev difference between SBObs and MCsim


t_data_org = reshape(CMFs_RGB_SB, length(wv_SB)*3, size(CMFs_RGB_SB,3))'; 
t_data_prd = reshape(CMFs_RGB_VisM, length(wv_SB)*3, size(CMFs_RGB_VisM,3))'; 

t_stat_org = stat(t_data_org); 
t_stat_prd = stat(t_data_prd); 

idx_BelowThr = abs(t_stat_org(3,:))<0.003; % ignore noisy data

t_StdevPct_org = t_stat_org(6,:)./abs(t_stat_org(3,:))*100; 
t_StdevPct_prd = t_stat_prd(6,:)./abs(t_stat_prd(3,:))*100; 
t_StdevPct_org(idx_BelowThr) = []; 
t_StdevPct_prd(idx_BelowThr) = []; 

% Difference in StdDev for each wavelength
stdDevDiffPct = (t_stat_prd(6,:) - t_stat_org(6,:)) ./ t_stat_org(6,:) .*100; 

% res_SD_prd = [res_SD_prd mean(t_StdevPct_prd)]; 
res_SD_org = t_stat_org(6,:)'; 
res_SD_prd = [res_SD_prd t_stat_prd(6,:)']; 

end

fprintf('The predicted standard deviation is %.5f\n', mean(res_SD_prd)); 

stat(res_SD_org)
t_stat_prd = stat(res_SD_prd); 
stat(t_stat_prd(3,:)')

%%


% Two-sample F-test to test if the variances of two samples are equal
% test performed for each of 105 dimensions (35*3)
t_F1 = reshape(CMFs_RGB_SB, 35*3,size(CMFs_RGB_SB,3)); 
t_F2 = reshape(CMFs_RGB_VisM, 35*3,size(CMFs_RGB_VisM,3)); 
Ftest_alpha = 0.05; 
% [Ftest_h, Ftest_p, Ftest_ci, Ftest_stat] = vartest2(t_F1', t_F2', 'Alpha',0.05, 'Tail','both');
[Ftest_h, Ftest_p, Ftest_ci, Ftest_stat] = vartest2(t_F1', t_F2', Ftest_alpha, 'both');

fprintf('Variances of two samples are different for %d cases!\n', sum(Ftest_h)); 
fprintf('Variances of two samples are equal for %d cases!\n', 35*3-sum(Ftest_h)); 






